/********************************************************
  Xpress-BCL C Example Problems
  =============================

  file foliomiis.c
  ````````````````
  Modeling a MIP problem
  to perform portfolio optimization.

  Same model as in foliomip3.c.
  -- Infeasible model parameter values --
  -- Retrieving MIIS --

  (c) 2011-2024 Fair Isaac Corporation
      author: S.Heipcke, Mar. 2011
********************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "xprb.h"


#define MAXNUM 5                   /* Max. number of different assets */
#define MAXRISK 1.0/4              /* Max. investment into high-risk values */
#define MINREG 0.1                 /* Min. investment per geogr. region */
#define MAXREG 0.25                /* Max. investment per geogr. region */
#define MAXSEC 0.15                /* Max. investment per ind. sector */
#define MAXVAL 0.225               /* Max. investment per share */
#define MINVAL 0.1                 /* Min. investment per share */

#define DATAFILE "folio5.cdat"     /* File with problem data */
#define MAXENTRIES 10000


int NSHARES;                       /* Number of shares */
int NRISK;                         /* Number of high-risk shares */
int NREGIONS;                      /* Number of geographical regions */
int NTYPES;                        /* Number of share types */

double *RET;                       /* Estimated return in investment */
int *RISK;                         /* High-risk values among shares */
char **LOC;                        /* Geogr. region of shares */
char **SEC;                        /* Industry sector of shares */

char **SHARES_n;
char **REGIONS_n;
char **TYPES_n;

#include "readfoliodata.c_"

int main(int argc, char **argv)
{
 int s,r,t,i;
 XPRBprob prob;
 XPRBctr Risk,Return,Cap,Num;
 XPRBctr *MinReg, *MaxReg, *LimSec, LinkL, LinkU;
 XPRBvar *frac;                   /* Fraction of capital used per share */
 XPRBvar *buy;                    /* 1 if asset is in portfolio, 0 otherwise */
 XPRBctr *iisctr;
 XPRBvar *iisvar;
 int numv, numc, numiis;

 readdata(DATAFILE);               /* Data input from file */

 prob = XPRBnewprob("FolioMIP3inf");  /* Initialize a new problem in BCL */

/* Create the decision variables (including upper bounds for `frac') */
 frac = (XPRBvar*)malloc(NSHARES*sizeof(XPRBvar));
 buy = (XPRBvar*)malloc(NSHARES*sizeof(XPRBvar));
 for(s=0;s<NSHARES;s++)
 {
  frac[s] = XPRBnewvar(prob, XPRB_PL, XPRBnewname("frac(%s)",SHARES_n[s]), 0, MAXVAL);
  buy[s] = XPRBnewvar(prob, XPRB_BV, XPRBnewname("buy(%s)",SHARES_n[s]), 0, 1);
 }

/* Objective: total return */
 Return = XPRBnewctr(prob, "Return", XPRB_N);
 for(s=0;s<NSHARES;s++) XPRBaddterm(Return, frac[s], RET[s]);
 XPRBsetobj(prob,Return);          /* Set the objective function */

/* Limit the percentage of high-risk values */
 Risk = XPRBnewctr(prob, "Risk", XPRB_L);
 for(s=0;s<NRISK;s++) XPRBaddterm(Risk, frac[RISK[s]], 1);
 XPRBaddterm(Risk, NULL, MAXRISK);

/* Limits on geographical distribution */
 MinReg = (XPRBctr*)malloc(NREGIONS*sizeof(XPRBctr));
 MaxReg = (XPRBctr*)malloc(NREGIONS*sizeof(XPRBctr));
 for(r=0;r<NREGIONS;r++)
 {
  MinReg[r] = XPRBnewctr(prob, XPRBnewname("MinReg(%s)",REGIONS_n[r]), XPRB_G);
  MaxReg[r] = XPRBnewctr(prob, XPRBnewname("MaxReg(%s)",REGIONS_n[r]), XPRB_L);
  for(s=0;s<NSHARES;s++)
   if(LOC[r][s]>0)
   {
    XPRBaddterm(MinReg[r], frac[s], 1);
    XPRBaddterm(MaxReg[r], frac[s], 1);
   }
  XPRBaddterm(MinReg[r], NULL, MINREG);
  XPRBaddterm(MaxReg[r], NULL, MAXREG);
 }

/* Diversification across industry sectors */
 LimSec = (XPRBctr*)malloc(NTYPES*sizeof(XPRBctr));
 for(t=0;t<NTYPES;t++)
 {
  LimSec[t] = XPRBnewctr(prob, XPRBnewname("LimSec(%s)",TYPES_n[t]), XPRB_L);
  for(s=0;s<NSHARES;s++)
   if(SEC[t][s]>0) XPRBaddterm(LimSec[t], frac[s], 1);
  XPRBaddterm(LimSec[t], NULL, MAXSEC);
 }

/* Spend all the capital */
 Cap = XPRBnewctr(prob, "Cap", XPRB_E);
 for(s=0;s<NSHARES;s++) XPRBaddterm(Cap, frac[s], 1);
 XPRBaddterm(Cap, NULL, 1);

/* Limit the total number of assets */
 Num = XPRBnewctr(prob, "Num", XPRB_L);
 for(s=0;s<NSHARES;s++) XPRBaddterm(Num, buy[s], 1);
 XPRBaddterm(Num, NULL, MAXNUM);

/* Linking the variables */
 for(s=0;s<NSHARES;s++)
 {
  LinkU = XPRBnewctr(prob, XPRBnewname("LinkU(%s)",SHARES_n[s]), XPRB_L);
  XPRBaddterm(LinkU, frac[s], 1);
  XPRBaddterm(LinkU, buy[s], -MAXVAL);
  LinkL = XPRBnewctr(prob, XPRBnewname("LinkL(%s)",SHARES_n[s]), XPRB_G);
  XPRBaddterm(LinkL, frac[s], 1);
  XPRBaddterm(LinkL, buy[s], -MINVAL);
 }

/* Solve the problem (MIP) */
 XPRBsetsense(prob, XPRB_MAXIM);
 XPRBmipoptimize(prob, "");

 if (XPRBgetmipstat(prob)==XPRB_MIP_INFEAS)
 {
  printf("MIP infeasible. Retrieving IIS.\n");

  numiis=XPRBgetnumiis(prob);     /* Get the number of independent IIS */
  printf("Number of IIS: %d\n", numiis);
  for(s=1;s<=numiis;s++)
  {
   XPRBgetmiis(prob, &iisvar, &numv, &iisctr, &numc, NULL, NULL, s);
   printf("IIS %d:  %d variables, %d constraints\n", s, numv, numc);
   if (numv>0)
   {                              /* Print all variables in the IIS */
    printf("        Variables: ");
    for(i=0;i<numv;i++) printf("%s ", XPRBgetvarname(iisvar[i]));
    printf("\n");
    XPRBfreemem(iisvar);          /* Free the array of variables */
   }
   if (numc>0)
   {                              /* Print all constraints in the IIS */
    printf("        Constraints: ");
    for(i=0;i<numc;i++) printf("%s ", XPRBgetctrname(iisctr[i]));
    printf("\n");
    XPRBfreemem(iisctr);          /* Free the array of constraints */
   }
  }
 }
 else
  printf("Feasible or search unfinished!\n");

 return 0;
}
